/*
	File name:	bmorphpgm.c
	Type:		C main file.
	Author:		Gang Liu
	Date:		11/06/97
	Function:	Perform some binary morphological operations on
			an input binary image and save the result to
			an output binary image.

			Non-zero pixels in the input image are taken to
			be the fore-ground pixels. In the output image,
			fore-ground pixels are marked by 255 and back-ground
			pixels are marked by 0.
*/
#include <stdio.h>

#include "must.h"
#include "pgm.h"
#include "bmorph.h"

#define	DISK	0
#define	SQUARE	1
#define	HLINE	2
#define	VLINE	3

void	usage(FILE *fp, char *cmd);

int	main(int argc, char *argv[])
{
	char	op;
	int	SEtype, SEsize, orig;
	char	*inname, *outname;

	int	xorig, yorig;
	BSE	se;

	uchar	**inimage, **outimage;
	int	width, height;

	int	i, j;

	if(argc<7) {
		usage(stderr, argv[0]);
		return -1;
	}

	inname=argv[1];
	i =sscanf(argv[2], "%c", &op);
	i+=sscanf(argv[3], "%d", &SEtype);
	i+=sscanf(argv[4], "%d", &SEsize);
	i+=sscanf(argv[5], "%d", &orig);
	outname=argv[6];
	if(i!=4) {
		usage(stderr, argv[0]);
		return -1;
	}

	if(	(op!='d' && op!='e' && op!='o' && op!='c') ||
		(SEtype!=DISK && SEtype!=SQUARE &&
		 SEtype!=HLINE && SEtype!=VLINE )	||
		(SEsize<0 || SEsize%2!=1) ||
		(orig!=0 && orig!=1)
	  ) myerror(-2, stderr, "Invalid input parameter(s).\n");

	xorig=yorig=0;
	if(orig==0) {
		xorig=yorig=SEsize/2;
		switch(SEtype) {
			case VLINE: xorig=0; break;
			case HLINE: yorig=0; break;
		}
	}

	switch(SEtype) {
		case DISK:
			i=disk_SE(SEsize, SEsize, xorig, yorig, SEsize/2, &se);
			break;
		case SQUARE:
			i=block_SE(SEsize, SEsize, xorig, yorig, &se);
			break;
		case HLINE:
			i=line_SE(SEsize, 1, xorig,yorig, 0,0, SEsize-1,0, &se);
			break;
		case VLINE:
			i=line_SE(1, SEsize, xorig,yorig, 0,0, 0,SEsize-1, &se);
			break;
	}
	if(i) myerror(-3, stderr, "Error making the structuring element.\n");

	i=ReadPGM(inname, &width, &height, &inimage);
	if(i) myerror(-4, stderr, "Error reading input PGM image file.\n");

	outimage=(uchar**)alloc2d(width, height, sizeof(uchar));
	if(outimage==NULL) myerror(-5, stderr, "Error allocating memory.\n");

	for(i=0;i<height;i++)
		for(j=0;j<width;j++)
			if(inimage[i][j]) inimage[i][j]=1;

	switch(op) {
		case 'd':
			bdilation(inimage, width, height, 1, 0, &se, outimage);
			break;
		case 'e':
			berosion(inimage, width, height, 1, 0, &se, outimage);
			break;
		case 'o':
			bopen(inimage, width, height, 1, 0, &se, outimage);
			break;
		case 'c':
			bclose(inimage, width, height, 1, 0, &se, outimage);
			break;
	}

	for(i=0;i<height;i++)
		for(j=0;j<width;j++)
			if(outimage[i][j]) outimage[i][j]=255;

	i=WritePGM(outname, width, height, outimage);
	if(i) myerror(-5, stderr, "Error saving output file.\n");

	free2d((void**)inimage);
	free2d((void**)outimage);

	return 0;
}

void	usage(FILE *fp, char *cmd)
{
fprintf(fp, "Function of %s:\n", cmd);
fprintf(fp,
"This utility takes an input PGM file as a binary image (non-zero pixels\n\
as fore-ground) and performs some binary morphological operations on it.\n\
The result is saved in a PGM file with 255 marking the fore-ground and\n\
0 marking the back-ground.\n");
fprintf(fp, "\n");
fprintf(fp, "Usage:\n\t %s <PGM1> <op> <SE> <SE size> <orig> <PGM2>\n", cmd);
fprintf(fp, "<PGM1> (char*) is the name of the input image file;\n");
fprintf(fp, "<op> (char) the morphological operation you want to perform:\n\
		'd' dilation; 'e' erosion; 'o' openning; 'c' closing.\n");
fprintf(fp, "<SE> (int) the structuring element you want to use:\n\
		0 for circular disk; 1 for square;\n\
		2 for horizontal line; 3 for vertical line;\n");
fprintf(fp, "<SE size> (int) size of the SE:\n\
		diameter (SE=0), side length (SE=1), length (SE=2, or 3);\n\
		should be an odd number;\n");
fprintf(fp, "<orig> (int) location of the SE's origin;\n\
		0 for center; 1 the upper-left corner;\n");
fprintf(fp, "<PGM2> (char*) is the name of the output image file.\n");
fprintf(fp, "\n");
}

