/*
	File name:	conncomppgm.c
	Type:		C main file.
	Author:		Gang Liu
	Date:		06/06/97
	Function:	Run connected component labeling on an input binary
			file in PGM format, and give the symbolic image
			file also in PGM format.
			
			Note:
			  1.  0 in the input image stands for the background,
				and all none-zero values treated as foreground;
			  2.  a macro WRAP is defined;
				if there are more than WRAP connected components
				in the input image, the symbols are wrapped
				back into the range between 1 and WRAP, with 0
				reserved for background.
*/
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <ctype.h>

#define	WRAP	255

#include "must.h"
#include "pgm.h"
#include "connect.h"

void	usage(char *cmd);

int	main(int argc, char *argv[])
{
	char	verbose;
	char	*fname;

	int	width, height;
	uchar	**image;

	int	neighbors;
	int	num_components;
	int	**labels, tmp;

	int	i, j;

	if(argc<4) {
		usage(argv[0]);
		return -1;
	}

	verbose=0;
	if(argc>=5 && toupper(argv[4][0])=='V') verbose=1;

	neighbors=atoi(argv[1]);
	if(neighbors!=4 && neighbors!=8) neighbors=8;

	fname=argv[2];
	i=ReadPGM(fname, &width, &height, &image);
	if(i) myerror(-2, stderr, "Error reading input image.\n");

	labels=(int**)alloc2d(width, height, sizeof(int));
	if(labels==NULL) myerror(-3, stderr, "Not enough memory.\n");

	i=connected_component(image[0], width, height, neighbors,
				labels[0], &num_components);
	if(i) myerror(i, stderr, "connected_component() failed.\n");

	for(i=0;i<height;i++)
		for(j=0;j<width;j++) {
			tmp=labels[i][j];
			if(tmp>WRAP) {
				tmp %= WRAP;
				if(!tmp) tmp=WRAP; /* multiples of WRAP */
			}
			image[i][j]=tmp;
		}

	fname=argv[3];
	i=WritePGM(fname, width, height, image);
	if(i) fprintf(stderr, "Error writing output.\n");

	if(verbose)
	printf("Using %d-connectivity and %d connected components found.\n",
		neighbors, num_components);

	if(num_components>WRAP)
		fprintf(stderr,
"Warning: number of connected components exceeded %d; extras wrapped.\n", WRAP);

	free2d((void**)image);
	free2d((void**)labels);

	if(i) i=-2;	/* signaling error in writing output file */
	return i;
}

void	usage(char *cmd)
{
fprintf(stderr, "Usage:\n\t %s <8|4> <PGM1> <PGM2> [v]\n", cmd);
fprintf(stderr, "<8|4>  use 8- or 4-connectivity.\n");
fprintf(stderr, "<PGM1> is input PGM file.\n");
fprintf(stderr, "<PGM2> is output PGM file.\n");
fprintf(stderr, "[v]    verbose.\n");
fprintf(stderr, "\n");
return;
}
