/*
	File name:	conncomppgm2raw.c
	Type:		C main file.
	Author:		Gang Liu
	Date:		08/21/97
	Function:	Run connected component labeling on an input binary
			file in PGM format, and give the symbolic image
			file raw data file, each pixel being represented
			a two bytes (unsigned short).
			
			Note:
			  1.  0 in the input image stands for the background,
				and all none-zero values treated as foreground;
			  2.  a macro WRAP is defined;
				if there are more than WRAP connected components
				in the input image, the symbols are wrapped
				back into the range between 1 and WRAP, with 0
				reserved for background.

			  3.  This is modified from conncomppgm.c.
*/
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <ctype.h>

#include "must.h"
#include "pgm.h"
#include "connect.h"

typedef	unsigned short	outtype;
#define	WRAP	65535

void	usage(char *cmd);

int	main(int argc, char *argv[])
{
	char	verbose;
	char	*fname;
	FILE	*fp;

	int	width, height;
	uchar	**image;

	int	neighbors;
	int	num_components;
	int	**labels;

	outtype	**outlabels, pixel;

	int	i, j;

	if(argc<4) {
		usage(argv[0]);
		return -1;
	}

	verbose=0;
	if(argc>=5 && toupper(argv[4][0])=='V') verbose=1;

	neighbors=atoi(argv[1]);
	if(neighbors!=4 && neighbors!=8) neighbors=8;

	fname=argv[2];
	i=ReadPGM(fname, &width, &height, &image);
	if(i) myerror(-2, stderr, "Error reading input image.\n");

	labels=(int**)alloc2d(width, height, sizeof(int));
	if(labels==NULL) myerror(-3, stderr, "Not enough memory.\n");

	i=connected_component(image[0], width, height, neighbors,
				labels[0], &num_components);
	if(i) myerror(i, stderr, "connected_component() failed.\n");
	free2d((void**)image);

	outlabels=(outtype**)alloc2d(width, height, sizeof(outtype));
	if(outlabels==NULL) myerror(-3, stderr, "Not enough memory.\n");

	for(i=0;i<height;i++)
		for(j=0;j<width;j++) {
			pixel=labels[i][j];
			if(pixel>WRAP)
				pixel = (pixel % WRAP) + 1;
			outlabels[i][j]=pixel;
		}

	fname=argv[3];
	i=0;
	fp=fopen(fname, "wb");
	if(fp==NULL) i=1;
	if(!i) {
		j=fwrite(outlabels[0], width*sizeof(outtype), height, fp);
		if(j!=height) i=1;
		i|=fclose(fp);
	}
	if(i) fprintf(stderr, "Error writing output.\n");

	if(verbose)
	printf("Using %d-connectivity and %d connected components found.\n",
		neighbors, num_components);

	if(num_components>WRAP)
		fprintf(stderr,
"Warning: number of connected components exceeded %d; extras wrapped.\n", WRAP);

	free2d((void**)labels);
	free2d((void**)outlabels);

	if(i) i=-2;	/* signaling error in writing output file */
	return i;
}

void	usage(char *cmd)
{
fprintf(stderr, "Usage:\n\t %s <8|4> <PGM> <BIN> [v]\n", cmd);
fprintf(stderr, "<8|4>  use 8- or 4-connectivity.\n");
fprintf(stderr, "<PGM> is input PGM file.\n");
fprintf(stderr, "<BIN> is output binary data file (2 bytes per pixel).\n");
fprintf(stderr, "[v]    verbose.\n");
fprintf(stderr, "\n");
return;
}
